/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse implementiert eine Objekt vom Typ 'typ.fahrStreifen'.<p>
 * 
 * Die Klasse dient zur Verwaltung der fahrstreifendaten.
 * 
 * ber Getter und Setter kann auf die Daten zugegriffen werden.
 *
 * @version $Revision: 1.5 $ / $Date: 2015/09/02 12:49:38 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class FahrStreifen implements ClientReceiverInterface
{
	/**
   * Stellt Methoden fr die Debugausgabe zur Vefgung.
   */
	private Debug m_debug = Debug.getLogger();
	/**
	 * Aktuelle Verbindung zum Datenverteiler.
	 */
	private ClientDavConnection m_connection;
	/**
	 * Zugriff auf das aktuelle Datenmodell.
	 */
	private DataModel m_dataModel;
	/**
	 * Systemobjekt des Fahrstreifen.
	 */
	private SystemObject m_objekt;	
	/**
	 * Referenz auf eine Fahrstreifen DE.
	 */
	private SystemObject m_fahrStreifenQuelle;	
	/**
	 * Lage des Fahrstreifen (HFS, 1FS usw).
	 */
	private String m_lage;
	/**
	 * Typ des Fahrstreifen (HauptFahrStreifen, berholFahrStreifen usw.).
	 */
	private String m_typ;
	/**
	 * Attributgruppe fr den Datenempfang Kurzzeitdaten Intervall Fahrstreifen.
	 */
	private AttributeGroup m_atgVerkehrsDatenKurzZeitIntervall;
	/**
	 * Attributgruppe fr den Datenempfang Kurzzeitdaten Fahrstreifen.
	 */
	private AttributeGroup m_atgVerkehrsDatenKurzZeitFs;
	/**
	 * Attributgruppe fr den Datenempfang Prognosedaten Fahrstreifen.
	 */
	private AttributeGroup m_atgVerkehrsDatenKurzZeitTrendextraPolationFs;
	/**
	 * Attributgruppe fr den Datenempfang KFZ Einzelfahrzeugdaten Fahrstreifen.
	 */
	private AttributeGroup m_atgKfzEinzelDatenFs;
	/**
	 * Attributgruppe fr den Datenempfang Passivierung Fahrstreifen.
	 */
	private AttributeGroup m_atgPassivierungFs;
	/**
	 * Aspekt fr den Datenempfang von Analysewerte.
	 */
	private Aspect m_aspExterneErfassung;
	/**
	 * Aspekt fr den Datenempfang von Analysewerte.
	 */
	private Aspect m_aspAnalyse;
	/**
	 * Aspekt fr den Datenempfang von Intervallwerte.
	 */
	private Aspect m_aspIntervall;
	/**
	 * Aspekt fr den Datenempfang von Prognosewerte.
	 */
	private Aspect m_aspPrognose;		
	/**
	 * Aspekt fr den Datenempfang der Passivierung.
	 */
	private Aspect m_aspZustand;		
	/**
	 * Zugriff auf die Kurzzeitdaten.
	 */
	private VerkehrsDatenKurzZeitFs m_verkehrsDatenKurzZeitFs;
	/**
	 * Zugriff auf die Kurzzeitdaten Intervall.
	 */
	private VerkehrsDatenKurzZeitIntervall m_verkehrsDatenKurzZeitIntervall;	
	/**
	 * Zugriff auf die Einzelfahrzeugdaten.
	 */
	private KfzEinzelDatenFs m_kfzEinzelDatenFs = new KfzEinzelDatenFs();		
	/**
	 * Liste von empfangenen Kfz-Einzelfahrzeugdaten.
	 */
	private Vector<KfzEinzelDatenFs> m_kfzEinzelDatenFsV = new Vector<KfzEinzelDatenFs>();
	/**
	 * Kennung Fahrstreifen passiviert.
	 */
	private boolean m_passiviert = false;

	/**
	 * Konstruktor der Klasse mit bernahme der Parameter.<p>
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @param objekt Systemobjekt Fahrstreifen.
	 */
	public FahrStreifen(ClientDavInterface connection, SystemObject objekt)
	{
		this(connection, objekt, null);
	}
	/**
	 * Konstruktor der Klasse mit bernahme der Parameter.<p>
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @param objekt Systemobjekt Fahrstreifen.
	 */
	public FahrStreifen(ClientDavInterface connection, SystemObject objekt, Aspect aspektPrognose)
	{
		m_connection = (ClientDavConnection)connection;
		m_dataModel = m_connection.getDataModel();
		
		m_objekt = objekt;
		
		m_atgVerkehrsDatenKurzZeitIntervall  = m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitIntervall");
		m_atgVerkehrsDatenKurzZeitFs = m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitFs");
		m_atgVerkehrsDatenKurzZeitTrendextraPolationFs = m_dataModel.getAttributeGroup("atg.verkehrsDatenKurzZeitTrendExtraPolationFs");
		m_atgKfzEinzelDatenFs = m_dataModel.getAttributeGroup("atg.kfzEinzelDatenFs");
		m_atgPassivierungFs = m_dataModel.getAttributeGroup("atg.passivierungFs");
		
		m_aspIntervall = m_dataModel.getAspect("asp.messWertErsetzung");
		m_aspAnalyse = m_dataModel.getAspect("asp.analyse");
		m_aspPrognose = m_dataModel.getAspect("asp.prognoseNormal");
		if (aspektPrognose != null)
			m_aspPrognose = aspektPrognose;
		
		m_aspExterneErfassung = m_dataModel.getAspect("asp.externeErfassung");
		m_aspZustand = m_dataModel.getAspect("asp.zustand");
		
		konfigurationsDatenBestimmen();
	}
	

	/**
	 * Diese Methode bestimmt die Konfigurationsdaten aus der Attributgruppe 'atg.fahrStreifen'
	 * und schreibt die Werte in die interne Struktur.
	 */
	private void konfigurationsDatenBestimmen()
	{
		if (m_objekt != null)
		{
			ConfigurationObject co = (ConfigurationObject)m_objekt;
			Data data = co.getConfigurationData(m_dataModel.getAttributeGroup("atg.fahrStreifen"));
			if (data != null)
			{
				m_fahrStreifenQuelle = data.getReferenceValue("FahrStreifenQuelle").getSystemObject();
				m_lage = data.getScaledValue("Lage").getValueText();
				m_typ = data.getScaledValue("Typ").getValueText();
			}
		}		
	}


	/**
	 * Anmelden der entsprechenden Daten zum Empfang am Datenverteiler.
	 */
	public void datenDavAnmelden()
	{
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgVerkehrsDatenKurzZeitIntervall, m_aspIntervall);
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgVerkehrsDatenKurzZeitFs, m_aspAnalyse);
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgVerkehrsDatenKurzZeitTrendextraPolationFs, m_aspPrognose);
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgKfzEinzelDatenFs, m_aspExterneErfassung);
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgPassivierungFs, m_aspZustand);
	}
	
	/**
	 * Abmelden der angemeldeten Daten am Datenverteiler.
	 */
	public void datenDavAbmelden()
	{
		empfaengerAbmeldenObjekt(this, m_objekt, m_atgVerkehrsDatenKurzZeitIntervall, m_aspIntervall);
		empfaengerAbmeldenObjekt(this, m_objekt, m_atgVerkehrsDatenKurzZeitFs, m_aspAnalyse);
		empfaengerAbmeldenObjekt(this, m_objekt, m_atgVerkehrsDatenKurzZeitTrendextraPolationFs, m_aspPrognose);
		empfaengerAbmeldenObjekt(this, m_objekt, m_atgKfzEinzelDatenFs, m_aspExterneErfassung);
		empfaengerAbmeldenObjekt(this, m_objekt, m_atgPassivierungFs, m_aspZustand);
	}
	
	/**
   * Fr ein Objekt Attributgruppe/Aspekt Kombination am Datenverteiler anmelden.
   * 
   * @param instanz Instanz.
   * @param objekt Objekt.
   * @param atg Attributgruppe.
   * @param asp Aspekt.
   * @return true = Anmeldung erfolgreich, false = nicht erfolgreich.
   */
  private boolean empfaengerAnmeldenObjekt(ClientReceiverInterface instanz, SystemObject objekt, AttributeGroup atg, Aspect asp)
  {
    DataDescription dd;
    boolean ok = false;

    m_debug.fine(("DatenAnmeldenObjekt " + atg.getPid() + " " + asp.getPid() + " fr " + objekt));
    
    try
    {  
    	dd = new DataDescription(atg, asp);

    	m_connection.subscribeReceiver(instanz, objekt, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
        
    	ok = true;
    } 
    catch (DataNotSubscribedException e)
    {
      m_debug.error(("Daten nicht zum Empfangen angemeldet " + e.getMessage()));
    } 
    
    return (ok);
  }
	/**
   * Fr ein Objekt Attributgruppe/Aspekt Kombination am Datenverteiler anmelden.
   * 
   * @param instanz Instanz.
   * @param objekt Objekt.
   * @param atg Attributgruppe.
   * @param asp Aspekt.
   * @return true = Anmeldung erfolgreich, false = nicht erfolgreich.
   */
  private boolean empfaengerAbmeldenObjekt(ClientReceiverInterface instanz, SystemObject objekt, AttributeGroup atg, Aspect asp)
  {
    DataDescription dd;
    boolean ok = false;

    m_debug.fine(("DatenAbmeldenObjekt " + atg.getPid() + " " + asp.getPid() + " fr " + objekt));
    System.out.println(("DatenAbmeldenObjekt " + atg.getPid() + " " + asp.getPid() + " fr " + objekt));
    
  	try
		{
			dd = new DataDescription(atg, asp);

			m_connection.unsubscribeReceiver(instanz, objekt, dd);
			  
			ok = true;
		}
		catch (Exception e)
		{
			m_debug.error(("Daten konnten nicht abgemeldet werden " + e.getMessage()));
		}
    
    return (ok);
  }

	/* (non-Javadoc)
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	@Override
	public void update(ResultData[] results)
	{
		for (int i = 0; i < results.length; i++)
		{
			ResultData resultData = results[i];

			// Intervalldaten
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgVerkehrsDatenKurzZeitIntervall))
    	{ 
    		if (m_verkehrsDatenKurzZeitIntervall == null)
    			m_verkehrsDatenKurzZeitIntervall = new VerkehrsDatenKurzZeitIntervall(resultData);
    		else
    			m_verkehrsDatenKurzZeitIntervall.setDaten(resultData);

//    		System.out.println("Kurzzeitdaten Intervall FS -> " + resultData.getObject().getPid() + " " + m_verkehrsDatenKurzZeitIntervall);
    	}

			// Kurzzeitdaten
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgVerkehrsDatenKurzZeitFs))
    	{ 
    		if (m_verkehrsDatenKurzZeitFs == null)
    			m_verkehrsDatenKurzZeitFs = new VerkehrsDatenKurzZeitFs(resultData);
    		else
    			m_verkehrsDatenKurzZeitFs.setDaten(resultData);

//    		System.out.println("Kurzzeitdaten FS -> " + resultData.getObject().getPid() + " " + m_verkehrsDatenKurzZeitFs);
    	}

    	// Prognosedaten
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgVerkehrsDatenKurzZeitTrendextraPolationFs))
    	{ 
//    		System.out.println("Prognose FS -> " + resultData);
    	}
    	
    	// Kfz Einzeldaten
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgKfzEinzelDatenFs))
    	{ 
    		System.out.println("Kfz Einzelfahrzeugdaten FS -> " + resultData);
    		
    		m_kfzEinzelDatenFs.auswerteAttributgruppe(resultData);
//    		System.out.println(m_objekt.getPid() + " " + m_kfzEinzelDatenFs);
    		    		
    		listeKfzEinzelDatenVerwalten(resultData, 50);
    		    		
    		m_kfzEinzelDatenFs.benachrichtigeListener();
    	}
    	
    	// Passivierung
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgPassivierungFs))
    	{ 
//    		System.out.println("Passivierung FS -> " + resultData);
    		
    		if (resultData.hasData())
    		{
    			Data data = resultData.getData();
    			
    			m_passiviert = data.getScaledValue("passiviert").getValueText().equals("Ja");
    		}
    		else
    		{
    			m_passiviert = false;
    		}
    	}

		}		
	}

	/**
	 * Diese Methode verwaltet die Liste der empfangenen KFZ-Einzeldaten Fahrstreifen.
	 * Es werden maximal Datenstze bis zum Erreichen von 'maxEintraege' vorgehalten. 
	 * Der neueste Datensatz wird an Index 0 eingetragen.
	 * 
	 * @param resultData empfangener Datensatz Kfz-Einzeldaten.
	 * @param maxEintraege maximale Anzahl der Eintrge.
	 */
	private void listeKfzEinzelDatenVerwalten(ResultData resultData, int maxEintraege)
	{
		KfzEinzelDatenFs _kfzEinzelDatenFs = new KfzEinzelDatenFs();

		_kfzEinzelDatenFs.auswerteAttributgruppe(resultData);
		
		if (m_kfzEinzelDatenFsV != null)
		{
			m_kfzEinzelDatenFsV.add(0, _kfzEinzelDatenFs);
			
			if (m_kfzEinzelDatenFsV.size() > maxEintraege)
			{
				m_kfzEinzelDatenFsV.setSize(maxEintraege);
			}
		}		
	}


	/**
	 * Gibt die Kurzzeitdaten des Fahrstreifen zurck.
	 * 
	 * @return Kurzzeitdaten des Fahrstreifen.
	 */
	public VerkehrsDatenKurzZeitFs getVerkehrsDatenKurzZeitFs()
	{
		return m_verkehrsDatenKurzZeitFs;
	}

	/**
	 * Gibt die Kurzzeitdaten Intervall des Fahrstreifen zurck.
	 * 
	 * @return Kurzzeitdaten Intervall des Fahrstreifen.
	 */
	public VerkehrsDatenKurzZeitIntervall getVerkehrsDatenKurzZeitIntervall()
	{
		return m_verkehrsDatenKurzZeitIntervall;
	}


	/**
	 * Gibt die Referenz auf eine Fahrstreifen DE zurck.
	 * @return Referenz auf eine Fahrstreifen DE.
	 */
	public SystemObject getFahrStreifenQuelle()
	{
		return m_fahrStreifenQuelle;
	}


	/**
	 * Gibt die Lage des Fahrstreifen (HFS, 1FS usw) zurck.
	 * @return Lage des Fahrstreifen (HFS, 1FS usw).
	 */
	public String getLage()
	{
		return m_lage;
	}

	/**
	 * Gibt den Index der Lage des Fahrstreifen (HFS=0, 1FS=1 usw) zurck.
	 * @return Index der Lage des Fahrstreifen (HFS=0, 1FS=1 usw).
	 */
	public int getLageIndex()
	{
		if (m_lage != null)
		{
			if (m_lage.equals("HFS"))
			{
				return 0;
			}
			else if (m_lage.equals("1FS"))
			{
				return 1;
			}
			else if (m_lage.equals("2FS"))
			{
				return 2;
			}
			else if (m_lage.equals("3FS"))
			{
				return 3;
			}
			else if (m_lage.equals("4FS"))
			{
				return 4;
			}
			else if (m_lage.equals("5FS"))
			{
				return 5;
			}
		}
		
		return -1;
	}


	/**
	 * Gibt den Typ des Fahrstreifen (HauptFahrStreifen, berholFahrStreifen usw.) zurck.
	 * @return Typ des Fahrstreifen (HauptFahrStreifen, berholFahrStreifen usw.).
	 */
	public String getTyp()
	{
		return m_typ;
	}


	/**
	 * Gibt das Objekt Fahrstreifen zurck.
	 * @return Objekt Fahrstreifen.
	 */
	public SystemObject getObjekt()
	{
		return m_objekt;
	}

	/**
	 * Gibt die Pid des Objekts zurck.
	 * @return Pid des Objekts.
	 */
	public String getPid()
	{
		if (m_objekt != null)
			return m_objekt.getPid();

		return null;
	}


	/**
	 * Gibt die KfzEinzeldatenFs zurck.
	 * 
	 * @return KfzEinzeldatenFs.
	 */
	public KfzEinzelDatenFs getKfzEinzelDatenFs()
	{
		return m_kfzEinzelDatenFs;
	}

	/**
	 * Lscht die empfangenen KFZ Einzeldaten Fahrstreifen.
	 */
	public void loescheKfzEinzelDatenFs()
	{
		m_kfzEinzelDatenFs.init();
	}
	
	/**
	 * Gibt die gesamte Liste der empfangenen KFZ-Einzeldaten zurck.
	 * 
	 * @return Liste der empfangenen KFZ-Einzeldaten.
	 */
	public List<KfzEinzelDatenFs> getKfzEinzelDatenFsAll()
	{
		List<KfzEinzelDatenFs> _liste = new ArrayList<KfzEinzelDatenFs>();
		
		_liste.addAll(m_kfzEinzelDatenFsV);
		
		return _liste;
	}

	/**
	 * Gibt 2 aufeinanderfolgende KFZ-Einzeldaten zurck.
	 * 
	 * @return Liste der empfangenen KFZ-Einzeldaten.
	 */
	public List<KfzEinzelDatenFs> getZweiAufeinanderFolgendeKfzEinzelDatenFs()
	{		
		if (m_kfzEinzelDatenFsV.size() >= 2)
		{
			List<KfzEinzelDatenFs> _liste = new ArrayList<KfzEinzelDatenFs>();

			_liste.add(m_kfzEinzelDatenFsV.lastElement());
			
			// letzten Datensatz lschen
			m_kfzEinzelDatenFsV.remove((m_kfzEinzelDatenFsV.size()-1));

			_liste.add(m_kfzEinzelDatenFsV.lastElement());
			
			return _liste;
		}
		
		return null;
	}


	/**
	 * Gibt den Wert der Variable m_passiviert zurck.
	 *
	 * @return Wert der Variable m_passiviert.
	 */
	public boolean isPassiviert()
	{
		return m_passiviert;
	}
}
