/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse zum Verwalten der SituationsEigenschaften.<p>
 * 
 * Folgende Werte werden in dieser Klasse gespeichert:<p>
 * 
 * - Startzeit in Millisekunden<br>
 * - Zeitdauer in Millisekunden<br>
 * - Startoffset in Zentimeter<br>
 * - Endoffset in Zentimeter<br>
 * - Liste SystemObjekte der StraenSegmente<p>
 *
 * ber die Methode init() knnen Default-Werte gesetzt werden.<br>
 * Der Zugriff auf die Inhalte erfolgt mit entsprechende Getter und Setter.
 *
 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class SituationsEigenschaften
{
	/**
	 * Startzeit in Millisekunden.
	 */
	private long m_startzeit;
	/**
	 * Zeitdauer in Millisekunden.
	 */
	private long m_dauer;
	
	/**
	 * Start-Offset in cm.
	 */
	private long m_startOffset;
	/**
	 * End-Offset in cm.
	 */
	private long m_endOffset;
	
	/**
	 * Liste der von dem Stau betroffenen StraenSegmente.
	 */
	private List<SystemObject> m_straenSegment = new ArrayList<SystemObject>();

	/**
	 * Lnge der Situation in cm.
	 */
	private long m_laenge;
	
	/**
	 * Default-Konstruktor ohne weitere Funktion.
	 */
	public SituationsEigenschaften()
	{		
	}
	
	/**
	 * Initialisierung der SituationsEigenschaften auf Defaultwerte.
	 */
	public void init()
	{
	  m_startzeit   = -1;
	  m_dauer       = -1;
	  m_startOffset = -1;
	  m_endOffset   = -1;
	  m_laenge   = 0;
	  m_straenSegment.clear();	  
	}
	
	/**
	 * Gibt die Dauer der Situation zurck.
	 * @return Dauer in Millisekunden.
	 */
	public long getDauer()
	{
		return m_dauer;
	}

	/**
	 * Setzt die Dauer in Millisekunden.
	 * 
	 * @param dauer Dauer in Millisekunden.
	 */
	public void setDauer(long dauer)
	{
		m_dauer = dauer;
	}

	/**
	 * Gibt den Endoffset in Zentimeter zurck.
	 * 
	 * @return Offset in Zentimeter.
	 */
	public long getEndOffset()
	{
		return m_endOffset;
	}
	
	/**
	 * Gibt den Endoffset in Meter zurck.
	 * 
	 * @return Offset in Meter.
	 */
	public long getEndOffsetMeter()
	{
		return m_endOffset/100;
	}

	/**
	 * Setzen des Offset in Zentimeter.
	 * 
	 * @param endOffset Offset in Zentimeter.
	 */
	public void setEndOffset(long endOffset)
	{
		m_endOffset = endOffset;
	}

	/**
	 * Setzen des Offset in Meter.
	 * 
	 * @param endOffset Offset in Meter.
	 */
	public void setEndOffsetMeter(long endOffset)
	{
		m_endOffset = endOffset*100;
	}

	/**
	 * Gibt den Startoffset in Zentimeter zurck.
	 * 
	 * @return Offset in Zentimeter
	 */
	public long getStartOffset()
	{
		return m_startOffset;
	}

	/**
	 * Gibt den Startoffset in Meter zurck.
	 * 
	 * @return Offset in Meter.
	 */
	public long getStartOffsetMeter()
	{
		return m_startOffset/100;
	}

	/**
	 * Setzen des Offset in Zentimeter.
	 * 
	 * @param startOffset Offset in Zentimeter
	 */
	public void setStartOffset(long startOffset)
	{
		m_startOffset = startOffset;
	}

	/**
	 * Setzen des Offset in Meter.
	 * 
	 * @param startOffset Offset in Meter.
	 */
	public void setStartOffsetMeter(long startOffset)
	{
		m_startOffset = startOffset*100;
	}

	/**
	 * Gibt die Startzeit zurck.
	 * @return Zeit in Millisekunden.
	 */
	public long getStartzeit()
	{
		return m_startzeit;
	}

	/**
	 * Setzt die Startzeit in Millisekunden.
	 * 
	 * @param startzeit Zeit in Millisekunden.
	 */
	public void setStartzeit(long startzeit)
	{
		m_startzeit = startzeit;
	}

	/**
	 * Gibt die Liste mit StraenSegmenten zurck.
	 * 
	 * @return Liste StraenSegmente.
	 */
	public List<SystemObject> getStraenSegment()
	{
		return m_straenSegment;
	}

	/**
	 * Setzt die Liste StraenSegmente.
	 * 
	 * @param straenSegment Liste StraenSegmente
	 */
	public void setStraenSegment(List<SystemObject> straenSegment)
	{
		m_straenSegment = straenSegment;
	}

	/**
	 * Fgt der Liste StraenSegment ein Element am Ende hinzu.
	 * 
	 * @param straenSegment Objekt StraenSegment
	 */
	public void addStraenSegmentToList(SystemObject straenSegment)
	{
		if (m_straenSegment == null)
			m_straenSegment = new ArrayList<SystemObject>();
		
		if (!m_straenSegment.contains(straenSegment))
			this.m_straenSegment.add(straenSegment);
	}

	/**
	 * Trgt das bergebene Straensegment an erster Stelle in die Liste der StraenSegmente ein.
	 * Die StraenSegmente die schon in der Liste waren, werden nachfolgend wieder in die Liste geschrieben.
	 * 
	 * @param straenSegment Objekt StraenSegment.
	 */
	public void addStraenSegmentToListFirst(SystemObject straenSegment)
	{
		if (m_straenSegment != null)
		{
			if (!m_straenSegment.contains(straenSegment))
				m_straenSegment.add(0, straenSegment);			
		}
	}

	/**
	 * Gibt die Lnge in Zentimeter zurck.
	 * 
	 * @return Lnge in Zentimeter.
	 */
	public long getLaenge()
	{
		return m_laenge;
	}
	
	/**
	 * Gibt die Lnge in Meter zurck.
	 * 
	 * @return Lnge in Meter.
	 */
	public long getLaengeMeter()
	{
		return m_laenge/100;
	}
	
	/**
	 * Setzt die lnge in Zentimeter.
	 * 
	 * @param laenge Lnge in Zentimeter.
	 */
	public void setLaenge(long laenge)
	{
		m_laenge = laenge;
	}	

	/**
	 * Setzt die lnge in Meter.
	 * 
	 * @param laenge Lnge in Meter.
	 */
	public void setLaengeMeter(long laenge)
	{
		m_laenge = laenge*100;
	}	
	
	/**
	 * Addiert die Lnge in Zentimeter zur aktuellen Staulnge hinzu.
	 * 
	 * @param laenge Lnge in Zentimter.
	 */
	public void addLaenge(long laenge)
	{
		m_laenge += laenge;
	}

	/**
	 * Addiert die Lnge in Zentimeter zur aktuellen Lnge hinzu.
	 * 
	 * @param laenge Lnge in Zentimter.
	 */
	public void addLaengeMeter(long laenge)
	{
		m_laenge += (laenge*100);
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString()
	{
		StringBuffer string = new StringBuffer();
		
		string.append("SituationsEigenschaften " +
				          " StartZeit " +  m_startzeit +
				          " Dauer " + m_dauer +
				          " StartOffset " + m_startOffset +
				          " EndOffset " + m_endOffset +
				          " Staulnge " + m_laenge + 
				          " StraenSegment " + m_straenSegment);
		
		return string.toString();
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj)
	{
		if (this == obj)
			return true;
		
		SituationsEigenschaften se = (SituationsEigenschaften)obj;
		if (se != null)
		{
			if (this.m_straenSegment.size() != se.getStraenSegment().size())
				return false;
			
			for (int i = 0; i < m_straenSegment.size(); i++)
			{
				if (!m_straenSegment.get(i).equals(se.getStraenSegment().get(i)))
					return false;
			}
			
			if (this.m_startOffset != se.getStartOffset())
				return false;
			if (this.m_endOffset != se.getEndOffset())
				return false;			
			if (this.m_laenge != se.getLaenge())
				return false;
			
			return true;
		}
				
		return false;
	}
}
