/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Adolf-Dambach-Strae
 * 76571 Gaggenau
 * Phone: +49-7225-64-01
 * Fax: +49-7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.nachlauf;

import java.text.SimpleDateFormat;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.stealgsba.algorithmen.lib.EinzelsituationVorstufe;
import de.bsvrz.ste.stealgsba.algorithmen.nachlauf.ParameterNachlauf.Reaktion;
import de.bsvrz.ste.stealgsba.algorithmusrahmen.UrsacheneinheitErweiterung;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse realisiert die Ursacheneinheit fr die Funktionalitt des Steuerungsalgorithmus 
 * "Nachlauf". 
 * 
 * @version $Revision: 1.4 $ / $Date: 2015/10/16 09:05:50 $ / ($Author: Asal $)
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class NachlaufUe extends UrsacheneinheitErweiterung implements ClientReceiverInterface
{
	/**
   * Stellt Methoden fr die Debugausgabe zur Vefgung.
   */
	private static Debug m_debug = Debug.getLogger();
	/**
	 * Systemobjekt der Ursacheneinheit.
	 */
	private SystemObject m_objekt;
	/**
	 * Attributgruppe fr den Empfang der globalen Parameter.
	 */
	private AttributeGroup m_atgParameter1;
	/**
	 * Aspekt fr den Empfang der globalen Parameter.
	 */
	private Aspect m_aspParameter;	
  /**
   * Parameter fr die Steuerungslogik Lrm.
   */
  private ParameterNachlauf m_paraNachlauf = new ParameterNachlauf();
  /**
   * Zugriff auf den Status der Einzelsituationvorstufe.
   */
  private EinzelsituationVorstufe m_einzelsituationVorstufe = null;  
  /**
   * Status der Ursacheneinheit.
   */
  private boolean m_statusEinzelSituationVorstufeAlt = false;
	/**
	 * Kennung, ob Nachlauf aktiviert ist.
	 */
  private boolean m_nachlaufAktiv = false;
	/**
	 * Endzeitpunkt fr den Nachlauf in Millisekunden.
	 */
  private long m_nachlaufZeitpunktEnde;
	/**
	 * Attributgruppe fr den Versand der Analysedaten der Situationsbestimmung.
	 */
	private AttributeGroup m_atgAnalyse;
	/**
	 * Aspekt fr den Versand der Analysedaten.
	 */
	private Aspect m_aspAnalyse;
	
	/**
	 * Konstruktor mit bernahme der Parameter.<p>
	 * Anmelden zum Empfang der bentigten Parameter am Datenverteiler.
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @param ue Ursacheneinheit.
	 * @param aspektVersand Aspekt fr den Versand.
	 */
	public NachlaufUe(ClientDavInterface connection, SystemObject ue, Aspect aspektVersand)
	{
		super(connection, ue, aspektVersand);

		m_objekt = ue;
		
		m_atgParameter1 = m_dataModel.getAttributeGroup("atg.sswSbaUeNachlauf");    
		m_aspParameter  = m_dataModel.getAspect("asp.parameterSoll");

		m_atgAnalyse = m_dataModel.getAttributeGroup("atg.sswSbaUeNachlaufAnalyse");
		m_aspAnalyse = m_dataModel.getAspect("asp.analyse");
		
		// Daten zum Empfang anmelden		
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgParameter1, m_aspParameter);

		// Daten zum Versand anmelden
		quelleAnmeldenObjekt(this, m_objekt, m_atgAnalyse, m_aspAnalyse);		
}
	
	/**
	 * Methode berechnet die aktuelle Situation der Ursacheneinheit.
	 */
	public boolean berechnen()
	{
		System.out.println("NachlaufUe.berechnen() -> " + m_objekt.getPid());

    SimpleDateFormat df = new SimpleDateFormat ("dd.MM.yyyy HH:mm:ss,SSS");

		boolean zurueck = false;
	  boolean einschaltKriterium = false;
	  boolean nachlaufEndeErreicht = false;
	  boolean statusNok = false;
	  
	  NachlaufAnalyse analyse = new NachlaufAnalyse();
	  
	  /**
	   * Status neu berechnen.
	   */
    if ((m_einzelsituationVorstufe != null) &&
    		(m_paraNachlauf != null))
    {
    	analyse.setUrsacheneinheit(m_paraNachlauf.getUrsacheneinheit());
    	analyse.setEinschaltdauer(m_paraNachlauf.getEinschaltdauer());
    	analyse.setReaktion(m_paraNachlauf.getReaktion());
    	analyse.setStatusInput(m_einzelsituationVorstufe.isStatus());
    	analyse.setNachlaufAktiv(m_nachlaufAktiv);
    	analyse.setNachlaufEnde(m_nachlaufZeitpunktEnde);
    	analyse.setStatus(this.getStatus());
    	
  	  if (m_nachlaufAktiv)
  	  {
  	  	/**
  	  	 * Prfe auf korrekten Status fr Nachlauf der Input-UE
  	  	 */
	    	if (prfeStatus(m_paraNachlauf.getReaktion(), m_einzelsituationVorstufe.isStatus()))
	    	{
		  	  /**
		  	   * Prfen, ob Nachlaufende erreicht, wenn nicht alten Status beibehalten.
		  	   */
		  	  if (!isNachlaufendeErreicht())
		  	  {
		  	  	System.out.println("NachlaufUe.berechnen() -> Nachlaufzeit noch aktiv Status=" + this.getStatus() + " beibehalten ZeitAkt: " + df.format(m_connection.getTime()) + " Ende Nachlauf: " + df.format(m_nachlaufZeitpunktEnde));
		  	  	
		  	  	sendeAnalyseDaten(analyse);
		  	  	
		  	  	return this.getStatus();
		  	  }
		  	  else
		  	  {
		  	  	nachlaufEndeErreicht = true;
		  	  }
	    	}
	    	else
	    	{
	    		statusNok = true;    		
	    	}
	    	
	    	/**
	    	 * Nachlauf abgelaufen oder Statusnderung, Flags lschen
	    	 */
	    	if ((statusNok) || (nachlaufEndeErreicht))
	    	{
	  			m_nachlaufAktiv = false;
	  			m_statusEinzelSituationVorstufeAlt = m_einzelsituationVorstufe.isStatus();	  	
	    	}    	
  	  }
  	  
    	/**
    	 * Prfe auf Statuswechsel UE.
    	 */
    	System.out.println("NachlaufUe.berechnen() -> UE-Input " + m_einzelsituationVorstufe.getUrsacheneinheit().getName() + " Status " + m_einzelsituationVorstufe.isStatus());

    	einschaltKriterium = prfeAktivierung(m_paraNachlauf.getReaktion(), m_einzelsituationVorstufe.isStatus());
    	
  		System.out.println("NachlaufUe.berechnen() -> einschaltKriterium " + einschaltKriterium + " Reaktion=" + m_paraNachlauf.getReaktion() + " StatusAlt " + m_statusEinzelSituationVorstufeAlt);
    	
  		if (einschaltKriterium)
  		{
  			m_nachlaufAktiv = true;

  			// Zeitpunkt des Nachlaufende setzen.
  			m_nachlaufZeitpunktEnde = m_connection.getTime() + m_paraNachlauf.getEinschaltdauer();
  			
  			zurueck = true;
  		}  		
    }
    else
    {
    	System.out.println("Fehlerhafte Parameterversorgung: " + m_paraNachlauf);
    	m_debug.fine("Fehlerhafte Parameterversorgung: " + m_paraNachlauf);
    }

    analyse.setStatus(zurueck);
    
    sendeAnalyseDaten(analyse);
    
    System.out.println("NachlaufUe.berechnen() -> " + m_objekt.getPid() + " Aktueller Status " + zurueck);
    
    return zurueck;
	}
	
	/**
	 * Diese Methode versendet die Analysedaten der Situationsbestimmung ber den Datenverteiler.
	 * 
	 * @param analyse Klasse mit den Analysedaten.
	 */
	private void sendeAnalyseDaten(NachlaufAnalyse analyse)
	{
		DataDescription dd = new DataDescription(m_atgAnalyse, m_aspAnalyse);

		Data data_neu = analyse.erzeugeDatensatz((ClientDavConnection)m_connection);

		ResultData erg = new ResultData(m_objekt, dd, m_connection.getTime(), data_neu);
		
		try
		{
			m_connection.sendData(erg);
		}
		catch (DataNotSubscribedException e)
		{
			e.printStackTrace();
		}
		catch (SendSubscriptionNotConfirmed e)
		{
			e.printStackTrace();
		}
	}

	/**
	 * Prft, ob das Ende der Nachlaufzeit erreicht ist.
	 * 
	 * @return true wenn Ende der Nachlaufzeit erreicht, ansonsten false.
	 */
	private boolean isNachlaufendeErreicht()
	{
		if (m_connection.getTime() < m_nachlaufZeitpunktEnde)
		{
			return false;
		}
		
		return true;
	}
	
	/**
	 * Prft anhand der gegebenen Werte, ob die Ursacheneinheit aktiviert werden soll.
	 * 
	 * @param reaktion Reaktion (Ein-/Ausschaltung).
	 * @param status Aktuellen Status der Ursacheneinheit Input.
	 * @return true wenn die UE aktiviert werden soll, ansonsten false.
	 */
	private boolean prfeAktivierung(Reaktion reaktion, boolean status)
	{
		if (reaktion.equals(Reaktion.AUSSCHALTUNG))
		{
			if ((m_statusEinzelSituationVorstufeAlt) && (!status))
			{
				return true;
			}
		}
		else if (reaktion.equals(Reaktion.EINSCHALTUNG))
		{
			if ((!m_statusEinzelSituationVorstufeAlt) && (status))
			{
				return true;
			}			
		}
		
		m_statusEinzelSituationVorstufeAlt = status;
		
		return false;
	}

	/**
	 * Prft anhand der gegebenen Werte, ob der Status fr eine Aktivierung des Nachlauf OK ist.
	 * 
	 * @param reaktion Reaktion (Ein-/Ausschaltung).
	 * @param status Aktuellen Status der Ursacheneinheit Input.
	 * @return true wenn der Status fr eine Aktivierung ok, ansonsten false.
	 */
	private boolean prfeStatus(Reaktion reaktion, boolean status)
	{
		if (reaktion.equals(Reaktion.AUSSCHALTUNG))
		{
			if (!status)
			{
				return true;
			}
		}
		else if (reaktion.equals(Reaktion.EINSCHALTUNG))
		{
			if (status)
			{
				return true;
			}			
		}
		
		return false;
	}

	/* (non-Javadoc)
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	public void update(ResultData results[])
	{
		System.out.println("NachlaufUe.update()");
		
		for (int i = 0; i < results.length; i++)
		{
			ResultData resultData = results[i];
			
      if (resultData.getDataDescription().getAttributeGroup().equals(m_atgParameter1))
      { 
//        System.out.println("-------> " + resultData);
        
        m_paraNachlauf.setdaten(resultData);

        System.out.println(m_paraNachlauf);
        m_debug.fine(m_paraNachlauf.toString());
        
        erzeugeObjektEinzelsituationVorstufe();
      }
      
		}
	}

	/**
	 * Diese Methode erzeugt das Objekt fr die Verwaltung der Einzelsituationvorstufe entsprechend
	 * dem empfangenen Parameter.
	 */
	private void erzeugeObjektEinzelsituationVorstufe()
	{
		if (m_paraNachlauf != null)
		{
			SystemObject ue = m_paraNachlauf.getUrsacheneinheit();
			if (ue != null)
			{
				if (m_einzelsituationVorstufe == null)
				{
					m_einzelsituationVorstufe = new EinzelsituationVorstufe((ClientDavConnection)m_connection, ue);
				}
				else
				{
					if (!m_einzelsituationVorstufe.getUrsacheneinheit().equals(ue))
					{
						m_einzelsituationVorstufe.abmeldeDav();
	
						m_einzelsituationVorstufe = new EinzelsituationVorstufe((ClientDavConnection)m_connection, ue);
					}
				}
			}
		}
	}

}
